@@ -4,6 +4,7 @@
 
 import itertools as itt
 import os
+import typing
 from collections import Counter
 from typing import Iterable, Mapping, Optional
 
@@ -18,23 +19,32 @@ __all__ = [
 @click.command()
 def export():
     """Create export data file."""
-    from biomappings.resources import load_mappings, load_predictions, load_false_mappings
+    from biomappings.resources import (
+        load_false_mappings,
+        load_mappings,
+        load_predictions,
+        load_unsure,
+    )
     from biomappings.utils import DATA
 
     path = os.path.join(DATA, "summary.yml")
 
     true_mappings = load_mappings()
     false_mappings = load_false_mappings()
+    unsure_mappings = load_unsure()
     rv = {
         "positive": _get_counter(true_mappings),
         "negative": _get_counter(false_mappings),
+        "unsure": _get_counter(unsure_mappings),
         "predictions": _get_counter(load_predictions()),
-        "contributors": _get_contributors(itt.chain(true_mappings, false_mappings)),
+        "contributors": _get_contributors(
+            itt.chain(true_mappings, false_mappings, unsure_mappings)
+        ),
     }
     rv.update(
         {
             f"{k}_mapping_count": sum(e["count"] for e in rv[k])
-            for k in ("positive", "negative", "predictions")
+            for k in ("positive", "negative", "unsure", "predictions")
         }
     )
     rv.update(
@@ -42,7 +52,7 @@ def export():
             f"{k}_prefix_count": len(
                 set(itt.chain.from_iterable((e["source"], e["target"]) for e in rv[k]))
             )
-            for k in ("positive", "negative", "predictions")
+            for k in ("positive", "negative", "unsure", "predictions")
         }
     )
     with open(path, "w") as file:
@@ -50,7 +60,7 @@ def export():
 
 
 def _get_counter(mappings: Iterable[Mapping[str, str]]):
-    counter = Counter()
+    counter: typing.Counter[typing.Tuple[str, str]] = Counter()
     for mapping in mappings:
         source, target = mapping["source prefix"], mapping["target prefix"]
         if source > target:
@@ -76,6 +86,7 @@ def _get_contributors(mappings: Iterable[Mapping[str, str]]):
 def _get_source(source: str) -> Optional[str]:
     if source.startswith("orcid:"):
         return source[len("orcid:") :]
+    return None
 
 
 if __name__ == "__main__":
