@@ -5,10 +5,11 @@
 import getpass
 import os
 from collections import defaultdict
-from typing import Any, Iterable, Mapping, Optional, Tuple
+from typing import Any, Dict, Iterable, List, Mapping, Optional, Set, Tuple
 
 import flask
 import flask_bootstrap
+from bioregistry.resolve_identifier import get_bioregistry_iri
 from flask_wtf import FlaskForm
 from wtforms import StringField, SubmitField
 
@@ -20,12 +21,20 @@ from biomappings.resources import (
     load_predictions,
     write_predictions,
 )
-from biomappings.utils import MiriamValidator, commit, not_main, push
+from biomappings.utils import (
+    check_valid_prefix_id,
+    commit,
+    get_branch,
+    get_curie,
+    not_main,
+    push,
+)
 
 app = flask.Flask(__name__)
 app.config["WTF_CSRF_ENABLED"] = False
 app.config["SECRET_KEY"] = os.urandom(8)
 app.config["SHOW_RELATIONS"] = True
+app.config["SHOW_LINES"] = False
 flask_bootstrap.Bootstrap(app)
 
 # A mapping from your computer's user, returned by getuser.getpass()
@@ -39,17 +48,21 @@ def _manual_source():
     return "web"
 
 
-miriam_validator = MiriamValidator()
-
-
 class Controller:
     """A module for interacting with the predictions and mappings."""
 
-    def __init__(self):  # noqa: D107
+    def __init__(self, target_curies: Optional[Iterable[Tuple[str, str]]] = None):
+        """Instantiate the web controller.
+
+        :param target_curies: Pairs of prefix, local unique identifiers that are the target
+            of curation. If this is given, pre-filters will be made before on predictions
+            to only show ones where either the source or target appears in this set
+        """
         self._predictions = load_predictions()
-        self._marked = {}
+        self._marked: Dict[int, str] = {}
         self.total_curated = 0
-        self._added_mappings = []
+        self._added_mappings: List[Dict[str, str]] = []
+        self.target_ids = set(target_curies or [])
 
     def predictions(
         self,
@@ -57,17 +70,32 @@ class Controller:
         offset: Optional[int] = None,
         limit: Optional[int] = None,
         query: Optional[str] = None,
+        source_query: Optional[str] = None,
+        target_query: Optional[str] = None,
         prefix: Optional[str] = None,
+        same_text: bool = False,
     ) -> Iterable[Tuple[int, Mapping[str, Any]]]:
         """Iterate over predictions.
 
         :param offset: If given, offset the iteration by this number
         :param limit: If given, only iterate this number of predictions.
-        :param query: If given, show only equivalences that have it appearing as a substring in one of the fields
-        :param prefix: If given, show only equivalences that have it appearing as a substring in one of the prefixes
+        :param query: If given, show only equivalences that have it appearing as a substring in one of the source
+            or target fields.
+        :param source_query: If given, show only equivalences that have it appearing as a substring in one of the source
+            fields.
+        :param target_query: If given, show only equivalences that have it appearing as a substring in one of the target
+            fields.
+        :param prefix: If given, show only equivalences that have it appearing as a substring in one of the prefixes.
+        :param same_text: If true, filter to predictions with the same label
         :yields: Pairs of positions and prediction dictionaries
         """
-        it = self._help_it_predictions(query=query, prefix=prefix)
+        it = self._help_it_predictions(
+            query=query,
+            source=source_query,
+            target=target_query,
+            prefix=prefix,
+            same_text=same_text,
+        )
         if offset is not None:
             try:
                 for _ in range(offset):
@@ -85,50 +113,91 @@ class Controller:
     def count_predictions(
         self,
         query: Optional[str] = None,
+        source_query: Optional[str] = None,
+        target_query: Optional[str] = None,
         prefix: Optional[str] = None,
+        same_text: bool = False,
     ) -> int:
         """Count the number of predictions to check for the given filters."""
-        return sum(1 for _ in self._help_it_predictions(query=query, prefix=prefix))
+        it = self._help_it_predictions(
+            query=query,
+            source=source_query,
+            target=target_query,
+            prefix=prefix,
+            same_text=same_text,
+        )
+        return sum(1 for _ in it)
 
-    def _help_it_predictions(self, query: Optional[str] = None, prefix: Optional[str] = None):
-        it = enumerate(self._predictions)
-        if query is not None:
-            query = query.casefold()
+    def _help_it_predictions(
+        self,
+        query: Optional[str] = None,
+        source: Optional[str] = None,
+        target: Optional[str] = None,
+        prefix: Optional[str] = None,
+        same_text: bool = False,
+    ):
+        it: Iterable[Tuple[int, Mapping[str, Any]]] = enumerate(self._predictions)
+        if self.target_ids:
             it = (
-                (line, prediction)
-                for line, prediction in it
-                if any(
-                    query in prediction[x].casefold()
-                    for x in (
-                        "source identifier",
-                        "source name",
-                        "target identifier",
-                        "target name",
-                    )
-                )
+                (line, p)
+                for (line, p) in it
+                if (p["source prefix"], p["source identifier"]) in self.target_ids
+                or (p["target prefix"], p["target identifier"]) in self.target_ids
+            )
+
+        if query is not None:
+            it = self._help_filter(
+                query,
+                it,
+                {
+                    "source prefix",
+                    "source identifier",
+                    "source name",
+                    "target prefix",
+                    "target identifier",
+                    "target name",
+                },
+            )
+        if source is not None:
+            it = self._help_filter(
+                source, it, {"source prefix", "source identifier", "source name"}
+            )
+        if target is not None:
+            it = self._help_filter(
+                target, it, {"target prefix", "target identifier", "target name"}
             )
         if prefix is not None:
-            prefix = prefix.casefold()
+            it = self._help_filter(prefix, it, {"source prefix", "target prefix"})
+
+        if same_text:
             it = (
                 (line, prediction)
                 for line, prediction in it
-                if any(
-                    prefix in prediction[x].casefold() for x in ("source prefix", "target prefix")
-                )
+                if prediction["source name"].casefold() == prediction["target name"].casefold()
+                and prediction["relation"] == "skos:exactMatch"
             )
 
-        it = ((line, prediction) for line, prediction in it if line not in self._marked)
-        return it
+        rv = ((line, prediction) for line, prediction in it if line not in self._marked)
+        return rv
+
+    @staticmethod
+    def _help_filter(query: str, it, elements: Set[str]):
+        query = query.casefold()
+        return (
+            (line, prediction)
+            for line, prediction in it
+            if any(query in prediction[element].casefold() for element in elements)
+        )
 
     @staticmethod
     def get_curie(prefix: str, identifier: str) -> str:
         """Return CURIE for a given prefix and identifier."""
-        return miriam_validator.get_curie(prefix, identifier)
+        return get_curie(prefix, identifier)
 
     @classmethod
     def get_url(cls, prefix: str, identifier: str) -> str:
         """Return URL for a given prefix and identifier."""
-        return miriam_validator.get_url(prefix, identifier)
+        return get_bioregistry_iri(prefix, identifier)
 
     @property
     def total_predictions(self) -> int:
@@ -159,7 +228,7 @@ class Controller:
     ) -> None:
         """Add manually curated new mappings."""
         try:
-            miriam_validator.check_valid_prefix_id(source_prefix, source_id)
+            check_valid_prefix_id(source_prefix, source_id)
         except ValueError as e:
             flask.flash(
                 f"Problem with source CURIE {source_prefix}:{source_id}: {e.__class__.__name__}",
@@ -168,7 +237,7 @@ class Controller:
             return
 
         try:
-            miriam_validator.check_valid_prefix_id(target_prefix, target_id)
+            check_valid_prefix_id(target_prefix, target_id)
         except ValueError as e:
             flask.flash(
                 f"Problem with target CURIE {target_prefix}:{target_id}: {e.__class__.__name__}",
@@ -234,8 +303,12 @@ def home():
     limit = flask.request.args.get("limit", type=int, default=10)
     offset = flask.request.args.get("offset", type=int, default=0)
     query = flask.request.args.get("query")
+    source_query = flask.request.args.get("source")
+    target_query = flask.request.args.get("target")
     prefix = flask.request.args.get("prefix")
+    same_text = flask.request.args.get("same_text", default="false").lower() in {"true", "t"}
     show_relations = app.config["SHOW_RELATIONS"]
+    show_lines = app.config["SHOW_LINES"]
     return flask.render_template(
         "home.html",
         controller=controller,
@@ -243,8 +316,13 @@ def home():
         limit=limit,
         offset=offset,
         query=query,
+        source_query=source_query,
+        target_query=target_query,
         prefix=prefix,
+        same_text=same_text,
+        # configured
         show_relations=show_relations,
+        show_lines=show_lines,
     )
 
 
@@ -270,12 +348,18 @@ def add_mapping():
 @app.route("/commit")
 def run_commit():
     """Make a commit then redirect to the the home page."""
-    commit(
+    commit_info = commit(
         f'Curated {controller.total_curated} mapping{"s" if controller.total_curated > 1 else ""}'
         f" ({getpass.getuser()})",
     )
+    app.logger.warning("git commit res: %s", commit_info)
     if not_main():
-        push()
+        branch = get_branch()
+        push_output = push(branch_name=branch)
+        app.logger.warning("git push res: %s", push_output)
+    else:
+        flask.flash("did not push because on master branch")
+        app.logger.warning("did not push because on master branch")
     controller.total_curated = 0
     return _go_home()
 
@@ -312,8 +396,13 @@ def _go_home():
             limit=flask.request.args.get("limit", type=int),
             offset=flask.request.args.get("offset", type=int),
             query=flask.request.args.get("query"),
+            source=flask.request.args.get("source"),
+            target=flask.request.args.get("target"),
             prefix=flask.request.args.get("prefix"),
+            same_text=flask.request.args.get("same_text", default="false").lower() in {"true", "t"},
+            # config
             show_relations=app.config["SHOW_RELATIONS"],
+            show_lines=app.config["SHOW_LINES"],
         )
     )
 
